@@ -1,30 +1,29 @@
-from functools import partial
+import colorama
 import copy
 import flux
-import json
-import sys
-import socket
-from contextlib import contextmanager
-
 import gossip
-from logbook import Logger
-from sentinels import NOTHING
-from urlobject import URLObject as URL
-from vintage import deprecated, warn_deprecation
-
-import colorama
-
-from ... import _compat
-from ..._compat import httplib, iteritems, requests, RequestException, ProtocolError, unquote_url
+import http.client as httplib
+import json
+import requests
+import socket
+import sys
+import urllib3.exceptions
 from ..config import config
 from ..exceptions import (APICommandFailed, APITransportFailure,
-                          CommandNotApproved, SystemNotFoundException, MethodDisabled)
+                          CommandNotApproved, MethodDisabled,
+                          SystemNotFoundException)
 from .special_values import translate_special_values
+from contextlib import contextmanager
+from functools import partial
+from logbook import Logger
+from requests.exceptions import RequestException
+from sentinels import NOTHING
+from urllib.parse import unquote as unquote_url
+from urlobject import URLObject as URL
+from vintage import warn_deprecation
 
-
-_RETRY_REQUESTS_EXCEPTION_TYPES = (RequestException, socket.error, ProtocolError,
-                                   requests.packages.urllib3.exceptions.TimeoutError)
-
+_RETRY_REQUESTS_EXCEPTION_TYPES = (RequestException, socket.error, urllib3.exceptions.ProtocolError,
+                                   urllib3.exceptions.TimeoutError)
 _REQUESTS_HTTP_EXCEPTION_TYPES = (requests.exceptions.HTTPError, requests.models.HTTPError)
 
 _logger = Logger(__name__)
@@ -51,7 +50,7 @@ def _approval_preprocessor(approve, request):
         request.url = request.url.set_query_param('approved', str(approve).lower())
 
 
-class API(object):
+class API:
 
     def __init__(self, target, auth, use_ssl, ssl_cert):
         super(API, self).__init__()
@@ -71,7 +70,7 @@ class API(object):
         self._urls = [self._url_from_address(address, use_ssl) for address in target.get_api_addresses()]
         self._active_url = None
         self._checked_version = False
-        self._no_reponse_logs = 0 # Use counter instead of bool, improves support for coroutines
+        self._no_response_logs = 0 # Use counter instead of bool, improves support for coroutines
         self._use_pretty_json = config.root.api.log.pretty_json
         self._login_refresh_enabled = True
         self._disabled_http_methods = set()
@@ -152,7 +151,10 @@ class API(object):
 
     def __del__(self):
         if self._session is not None:
-            self._session.close()
+            try:
+                self._session.close()
+            except ReferenceError:
+                pass
 
     def reinitialize_session(self, auth=None):
         prev_auth = self._auth
@@ -304,10 +306,6 @@ class API(object):
         self._session.cookies.clear()
 
 
-    @deprecated(message="Use get_auth_context instead", since='46.0')
-    def auth_context(self, *args, **kwargs):
-        return self.get_auth_context(*args, **kwargs)
-
     get = _get_request_delegate("get")
     put = _get_request_delegate("put")
     post = _get_request_delegate("post")
@@ -348,6 +346,7 @@ class API(object):
         data = kwargs.pop("data", NOTHING)
         sent_json_object = None
         headers = kwargs.pop('headers', None)
+        files = kwargs.pop('files', None)
         if headers is None:
             headers = {}
         else:
@@ -379,7 +378,7 @@ class API(object):
 
             hostname = full_url.hostname
             api_request = requests.Request(http_method, full_url, data=data if data is not NOTHING else None,
-                                           params=url_params, headers=headers, auth=auth)
+                                           params=url_params, headers=headers, auth=auth, files=files)
             for preprocessor in self._preprocessors:
                 preprocessor(api_request)
 
@@ -398,9 +397,11 @@ class API(object):
             except _RETRY_REQUESTS_EXCEPTION_TYPES as e:  # pylint: disable=catching-non-exception
                 request_kwargs = dict(url=path, method=http_method, **kwargs)
                 _logger.debug('Exception while sending API command to {}: {}', self.system, e)
-                error_str = str(e)
-                if 'gaierror' in error_str or 'nodename nor servname' in error_str or \
-                    'Name or service not known' in error_str:
+                error_str = str(e).lower()
+                if any(substring in error_str for substring in ('gaierror',
+                                                                'nodename nor servname',
+                                                                'name or service not known',
+                                                                'temporary failure in name resolution')):
                     raise SystemNotFoundException(e, api_request, start_time)
                 raise APITransportFailure(self.system, request_kwargs, e, api_request, start_time)
 
@@ -411,7 +412,7 @@ class API(object):
             _logger.trace("{} --> {} {} (took {:.04f}s)", hostname, response.status_code, response.reason, elapsed)
             returned = Response(response, data, start_time, end_time)
             resp_data = returned.get_json()
-            if self._no_reponse_logs:
+            if self._no_response_logs:
                 logged_response_data = "..."
             elif self._use_pretty_json and resp_data is not None:
                 logged_response_data = json.dumps(resp_data, indent=4, separators=(',', ': '))
@@ -464,15 +465,11 @@ class API(object):
 
     @contextmanager
     def get_no_response_logs_context(self):
-        self._no_reponse_logs += 1
+        self._no_response_logs += 1
         try:
             yield
         finally:
-            self._no_reponse_logs -= 1
-
-    @deprecated(message="Use get_no_response_logs_context instead", since='46.0')
-    def no_response_logs_context(self):
-        return self.get_no_response_logs_context()
+            self._no_response_logs -= 1
 
     def add_auto_retry(self, retry_predicate, max_retries=1, sleep_seconds=None):
         if sleep_seconds is None: # backwards compatibility
@@ -554,7 +551,7 @@ class API(object):
         if sys.stdout.isatty():
             msg = colorama.Fore.YELLOW + msg + colorama.Fore.RESET
         # note: call through module to allow stubbing
-        return _compat.raw_input(msg).strip().lower() in ['yes', 'y']
+        return input(msg).strip().lower() in ['yes', 'y']
 
     def _is_approval_required(self, exception):
         if exception.response.get_error():
@@ -587,7 +584,7 @@ class API(object):
         return URL("{}://{}:{}".format("https" if use_ssl else "http", hostname, port)).add_path("/api/rest")
 
 
-class Response(object):
+class Response:
     """
     System API request response
     """
@@ -630,13 +627,16 @@ class Response(object):
         """
         return self._get_result()
 
+    def get_extra(self):
+        return self.get_json().get("extra")
+
     def get_error(self):
         """
         :returns: The error portion of the response as returned from the system, or None if it doesn't exist
         """
         json = self.get_json()
         if json is not None:
-            return json["error"]
+            return json.get("error")
 
     def __repr__(self):
         return repr(self.response)
@@ -666,7 +666,7 @@ class Response(object):
             raise APICommandFailed.raise_from_response(self)
 
 
-class _AutoRetryContext(object):
+class _AutoRetryContext:
     def __init__(self, global_retries_dict):
         self._retries_dict = None
         self._global_retries_dict = global_retries_dict
@@ -674,7 +674,7 @@ class _AutoRetryContext(object):
     def _should_retry_request(self, exc):
         if self._retries_dict is None:
             self._retries_dict = dict((k, v[0]) for k, v in self._global_retries_dict.items())
-        for retry_predicate, retries_left in iteritems(self._retries_dict):
+        for retry_predicate, retries_left in self._retries_dict.items():
             if retries_left < 1:
                 return None
             if retry_predicate not in self._global_retries_dict:
