@@ -2,17 +2,16 @@ import itertools
 import random
 from urlobject import URLObject as URL
 from numbers import Number
-from .._compat import xrange  # pylint: disable=redefined-builtin
 from .field import Field
 from .field_filter import FieldFilter
 from .q import QField
-from .exceptions import ObjectNotFound
+from .exceptions import ObjectNotFound, ChangedDuringIteration
 
 _DEFAULT_SYSTEM_PAGE_SIZE = 50
 _DEFAULT_PAGE_SIZE = 1000
 
 
-class QueryBase(object):
+class QueryBase:
 
     def count(self):
         return len(self)
@@ -35,7 +34,7 @@ class QueryBase(object):
         try:
             return self.sample(1)[0]
         except ValueError:
-            raise ObjectNotFound('No items where returned')
+            raise ObjectNotFound('No items were found')
 
     def sample(self, sample_count):
         """
@@ -47,12 +46,13 @@ class QueryBase(object):
         query_size = self.count()
         if query_size < sample_count:
             raise ValueError('Sample larger than returned items ({} > {})'.format(sample_count, query_size))
-        indexes = random.sample(xrange(query_size), sample_count)
+        indexes = random.sample(range(query_size), sample_count)
         return [self[index] for index in indexes]
 
 class LazyQuery(QueryBase):
-    def __init__(self, system, url):
+    def __init__(self, system, url, factory=None):
         super(LazyQuery, self).__init__()
+        self.factory = factory
         self.system = system
         self.query = url
         self._requested_page = None
@@ -61,25 +61,49 @@ class LazyQuery(QueryBase):
         self._objects_by_id = {}
         self._total_num_objects = None
         self._mutable = True
+        self._included_fields = None
+        self._extra = None
+
+    def get_extra(self):
+        self._fetch()
+        return self._extra
 
     def safe_get_by_id_from_cache(self, id): # pylint: disable=redefined-builtin
         return self._objects_by_id.get(id)
 
+    def extend_url(self, *args, **kw):
+        assert self._mutable, "Cannot modify query after fetching"
+        if args:
+            raise NotImplementedError("Positional arguments are not supported for LazyQuery")
+        url = URL(self.query)
+        if kw:
+            url = url.add_query_params(**kw)
+        self.query = url
+        return self
+
+    def include(self, fields):
+        if self._included_fields is None:
+            self._included_fields = fields
+        else:
+            self._included_fields.extend(fields)
+        fields_str = ','.join(fields)
+        return self.extend_url(include=fields_str)
+
     def __iter__(self):
         if self._total_num_objects is None:
             self._fetch()
         if self._requested_page is not None:
             start = (self._requested_page - 1) * self._requested_page_size
-            end = min(start + self._requested_page_size, len(self))
+            end = min(start + self._requested_page_size, self._total_num_objects)
         else:
             start = 0
             end = len(self)
-        for i in xrange(start, end):
+        for i in range(start, end):
             try:
                 yield self[i]
             except IndexError:
                 if i != self._total_num_objects:
-                    raise
+                    raise ChangedDuringIteration("Queried path's size changed during iteration")
 
     def __len__(self):
         if self._total_num_objects is None:
@@ -94,7 +118,14 @@ class LazyQuery(QueryBase):
         return self._total_num_objects % self._requested_page_size
 
     def _translate_item_if_needed(self, item_index):
-        pass
+        if self.factory is None:
+            return
+
+        received_item = self._fetched.get(item_index)
+        if isinstance(received_item, dict):
+            obj = self._fetched[item_index] = self.factory(self.system, received_item)
+            self._objects_by_id[obj.id] = obj
+
 
     def __getitem__(self, index):
         if isinstance(index, Number) and index < 0:
@@ -117,7 +148,8 @@ class LazyQuery(QueryBase):
 
             if self._total_num_objects is None:
                 self._total_num_objects = response.get_total_num_objects()
-
+            if self._included_fields is not None:
+                self._extra = response.get_extra()
             for index, obj in enumerate(response.get_result(), start=response.get_page_start_index()):
                 if self._fetched.get(index) is None:
                     self._fetched[index] = obj
@@ -166,23 +198,17 @@ class LazyQuery(QueryBase):
 
 class PolymorphicQuery(LazyQuery):
     def __init__(self, system, url, object_types, factory):
-        super(PolymorphicQuery, self).__init__(system, url)
+        super(PolymorphicQuery, self).__init__(system, url, factory)
         assert isinstance(object_types, (list, tuple)), "object_types must be tuple or list"
         self.object_types = object_types
-        self.factory = factory
-
-    def _translate_item_if_needed(self, item_index):
-        received_item = self._fetched.get(item_index)
-        if isinstance(received_item, dict):
-            obj = self._fetched[item_index] = self.factory(self.system, received_item)
-            self._objects_by_id[obj.id] = obj
+        assert callable(self.factory), "A callable factory must be provided for PolymorphicQuery"
 
     def _get_or_fabricate_field(self, field_name):
         for obj_type in self.object_types:
             field = obj_type.fields.get(field_name)
             if field is not None:
                 return field
-        self.object_types[0].fields.get_or_fabricate(field_name)
+        return self.object_types[0].fields.get_or_fabricate(field_name)
 
     def extend_url(self, *predicates, **kw):
         assert self._mutable, "Cannot modify query after fetching"
