@@ -1,32 +1,35 @@
 import copy
 import gossip
 import functools
-import gadget
-from contextlib import contextmanager
+import http.client as httplib
 
 from mitba import cached_method
+from sentinels import NOTHING
 from urlobject import URLObject as URL
+from contextlib import contextmanager
 
 from .exceptions import APICommandFailed
 from .system_object_utils import get_data_for_object_creation
-from .._compat import with_metaclass, iteritems, httplib, string_types  # pylint: disable=no-name-in-module
 from .exceptions import CacheMiss
 from api_object_schema import FieldsMeta as FieldsMetaBase
 from .field import Field
-from .type_binder import TypeBinder
+from .type_binder import TypeBinder, MonomorphicBinder
 from .bindings import PassthroughBinding
 from .api.special_values import translate_special_values
-from .utils import DONT_CARE, end_reraise_context
+from .utils import DONT_CARE, end_reraise_context, add_normalized_query_params
+from logbook import Logger
+
+_logger = Logger(__name__)
 
 
 class FieldsMeta(FieldsMetaBase):
 
     @classmethod
-    def FIELD_FACTORY(mcs, name):
+    def FIELD_FACTORY(mcs, name):  # pylint: disable=bad-mcs-classmethod-argument
         return Field(name, binding=PassthroughBinding())
 
 
-class BaseSystemObject(with_metaclass(FieldsMeta)):
+class BaseSystemObject(metaclass=FieldsMeta):
     """
     Base class for system objects and components
     """
@@ -35,7 +38,7 @@ class BaseSystemObject(with_metaclass(FieldsMeta)):
     URL_PATH = None
     UID_FIELD = "id"
     #: specifies which :class:`.TypeBinder` subclass is to be used for this type
-    BINDER_CLASS = TypeBinder
+    BINDER_CLASS = MonomorphicBinder
 
     def __init__(self, system, initial_data):
         super(BaseSystemObject, self).__init__()
@@ -44,6 +47,7 @@ class BaseSystemObject(with_metaclass(FieldsMeta)):
         self._cache = initial_data
         uid_field = self.fields[self.UID_FIELD]
         self._uid = uid_field.binding.get_value_from_api_object(system, type(self), self, self._cache)
+        self._use_cache_by_default = False
 
     @property
     def id(self):
@@ -71,6 +75,19 @@ class BaseSystemObject(with_metaclass(FieldsMeta)):
     def is_supported(cls, system): # pylint: disable=unused-argument
         return True
 
+    def is_in_system(self):
+        """
+        Returns whether or not the object actually exists
+        """
+        try:
+            self.get_field(self.UID_FIELD, from_cache=False)
+        except APICommandFailed as e:
+            if e.status_code != httplib.NOT_FOUND:
+                raise
+            return False
+        else:
+            return True
+
     @staticmethod
     def requires_cache_invalidation(*fields):
         invalidate_fields = fields
@@ -131,7 +148,20 @@ class BaseSystemObject(with_metaclass(FieldsMeta)):
         url_path = cls.URL_PATH
         if url_path is None:
             url_path = "/api/rest/{}".format(cls.get_plural_name())
-        return url_path
+        return URL(url_path)
+
+    @classmethod
+    def get_tags_for_object_operations(cls, system):
+        return [cls.get_type_name().lower(), system.get_type_name().lower()]
+
+    def safe_get_field(self, field_name, default=NOTHING, **kwargs):
+        """
+        Like :meth:`.get_field`, only returns 'default' parameter if no result was found
+        """
+        try:
+            return self.get_field(field_name, **kwargs)
+        except CacheMiss:
+            return default
 
     def get_field(self, field_name, from_cache=DONT_CARE, fetch_if_not_cached=True, raw_value=False):
         """
@@ -156,13 +186,16 @@ class BaseSystemObject(with_metaclass(FieldsMeta)):
         from_cache = self._deduce_from_cache(field_names, from_cache)
 
         if from_cache:
-            try:
+            if not fetch_if_not_cached:
                 return self._get_fields_from_cache(field_names, raw_value)
+            field_names_to_retrieve = field_names or \
+                [field.name for field in self.fields if self.system.is_field_supported(field)]
+            try:
+                return self._get_fields_from_cache(field_names_to_retrieve, raw_value)
             except CacheMiss:
-                if not fetch_if_not_cached:
-                    raise
+                pass
 
-        query = self.get_this_url_path()
+        query = self._get_fields_query()
 
         only_fields = []
         for field_name in field_names:
@@ -177,7 +210,7 @@ class BaseSystemObject(with_metaclass(FieldsMeta)):
 
         response = self.system.api.get(query)
 
-        result = response.get_result()
+        result = self._get_fields_result(response)
         self.update_field_cache(result)
 
         if not field_names:
@@ -185,6 +218,12 @@ class BaseSystemObject(with_metaclass(FieldsMeta)):
 
         return self._get_fields_from_cache(field_names, raw_value)
 
+    def _get_fields_query(self):
+        return self.get_this_url_path()
+
+    def _get_fields_result(self, response):
+        return response.get_result()
+
     def _is_caching_enabled(self):
         return self.system.is_caching_enabled()
 
@@ -195,8 +234,10 @@ class BaseSystemObject(with_metaclass(FieldsMeta)):
         if not field_names:
             return False
 
-        cache_enabled = self._is_caching_enabled()
+        if self._use_cache_by_default:
+            return True
 
+        cache_enabled = self._is_caching_enabled()
         for field_name in field_names:
             field = self.fields.get_or_fabricate(field_name)
             should_get_from_cache = field.cached
@@ -245,7 +286,7 @@ class BaseSystemObject(with_metaclass(FieldsMeta)):
         return self.system.is_field_supported(field)
 
     def update_field_cache(self, api_obj):
-        assert all(isinstance(key, string_types) for key in api_obj.keys())
+        assert all(isinstance(key, (str, bytes)) for key in api_obj.keys())
         self._cache.update(api_obj)
 
     def update_field(self, field_name, field_value):
@@ -265,7 +306,7 @@ class BaseSystemObject(with_metaclass(FieldsMeta)):
             'infinidat.sdk.pre_fields_update',
             {'source': self, 'fields': update_dict}, tags=hook_tags)
 
-        for field_name, field_value in list(iteritems(update_dict)):
+        for field_name, field_value in list(update_dict.items()):
             try:
                 field = self.fields[field_name]
             except LookupError:
@@ -283,7 +324,7 @@ class BaseSystemObject(with_metaclass(FieldsMeta)):
         except Exception as e:  # pylint: disable=broad-except
             with end_reraise_context():
                 gossip.trigger_with_tags('infinidat.sdk.object_update_failure',
-                                         {'obj': self, 'exception': e, 'system': self.system},
+                                         {'obj': self, 'exception': e, 'system': self.system, 'data': update_dict},
                                          tags=hook_tags)
         response_dict = res.get_result()
         if len(update_dict) == 1 and not isinstance(response_dict, dict):
@@ -319,7 +360,7 @@ class SystemObject(BaseSystemObject):
     """
     System object, that has query methods, creation and deletion
     """
-
+    BINDER_CLASS = TypeBinder
     @classmethod
     def find(cls, system, *predicates, **kw):
         binder = system.objects[cls.get_plural_name()]
@@ -331,30 +372,14 @@ class SystemObject(BaseSystemObject):
     def get_collection(self):
         return self.get_binder()
 
-    def is_in_system(self):
-        """
-        Returns whether or not the object actually exists
-        """
-        try:
-            self.get_field(self.UID_FIELD, from_cache=False)
-        except APICommandFailed as e:
-            if e.status_code != httplib.NOT_FOUND:
-                raise
-            return False
-        else:
-            return True
-
-    @classmethod
-    def get_tags_for_object_operations(cls, system):
-        return [cls.get_type_name().lower(), system.get_type_name().lower()]
-
     def _is_caching_enabled(self):
         return self.get_binder().is_caching_enabled() or super(SystemObject, self)._is_caching_enabled()
 
     @classmethod
-    def _create(cls, system, url, data, tags=None):
+    def _create(cls, system, url, data, tags=None, parent=None):
         hook_tags = tags or cls.get_tags_for_object_operations(system)
-        gossip.trigger_with_tags('infinidat.sdk.pre_object_creation', {'data': data, 'system': system, 'cls': cls},
+        gossip.trigger_with_tags('infinidat.sdk.pre_object_creation',
+                                 {'data': data, 'system': system, 'cls': cls, 'parent': parent},
                                  tags=hook_tags)
         try:
             returned = system.api.post(url, data=data).get_result()
@@ -362,11 +387,11 @@ class SystemObject(BaseSystemObject):
         except Exception as e:  # pylint: disable=broad-except
             with end_reraise_context():
                 gossip.trigger_with_tags('infinidat.sdk.object_creation_failure',
-                                         {'cls': cls, 'system': system, 'data': data, 'exception': e},
+                                         {'cls': cls, 'system': system, 'data': data, 'parent': parent, 'exception': e},
                                          tags=hook_tags)
         gossip.trigger_with_tags('infinidat.sdk.post_object_creation',
-                                 {'obj': obj, 'data': data, 'response_dict': returned}, tags=hook_tags)
-        gadget.log_entity_creation(entity=obj, params=data)
+                                 {'obj': obj, 'data': data, 'response_dict': returned, 'parent': parent},
+                                 tags=hook_tags)
         return obj
 
     @classmethod
@@ -397,6 +422,18 @@ class SystemObject(BaseSystemObject):
             for field in cls.fields  # pylint: disable=no-member
             if field.creation_parameter and not field.optional))
 
+    @contextmanager
+    def using_cache_by_default(self):
+        prev_value = self._use_cache_by_default
+        self._use_cache_by_default = True
+        _logger.debug('Entered use cache by default context for object {}', self)
+        try:
+            yield
+        finally:
+            self._use_cache_by_default = prev_value
+            _logger.debug('Exited use cache by default for object {}, use_cache new value is {}', self, prev_value)
+
+
     def safe_delete(self, *args, **kwargs):
         """
         Tries to delete the object, doing nothing if the object cannot be found on the system
@@ -407,26 +444,29 @@ class SystemObject(BaseSystemObject):
             if e.status_code != httplib.NOT_FOUND:
                 raise
 
-    def delete(self):
+    def delete(self, **kwargs):
         """
         Deletes this object.
         """
-        gadget.log_entity_deletion(self)
-        with self._get_delete_context():
-            self.system.api.delete(self.get_this_url_path())
+        self._send_delete_with_hooks_triggering(self.get_this_url_path(), **kwargs)
 
-    @contextmanager
-    def _get_delete_context(self):
+    def _send_delete_with_hooks_triggering(self, url, **kwargs):
+        url = add_normalized_query_params(url, **kwargs)
         hook_tags = self.get_tags_for_object_operations(self.system)
-        gossip.trigger_with_tags('infinidat.sdk.pre_object_deletion', {'obj': self}, tags=hook_tags)
+        gossip.trigger_with_tags('infinidat.sdk.pre_object_deletion', {'obj': self, 'url': url}, tags=hook_tags)
         try:
-            yield
+            resp = self.system.api.delete(url)
+            self._use_cache_by_default = True
         except Exception as e:       # pylint: disable=broad-except
             with end_reraise_context():
                 gossip.trigger_with_tags('infinidat.sdk.object_deletion_failure',
-                                         {'obj': self, 'exception': e, 'system': self.system},
+                                         {'obj': self, 'exception': e, 'system': self.system, 'url': url},
                                          tags=hook_tags)
-        gossip.trigger_with_tags('infinidat.sdk.post_object_deletion', {'obj': self}, tags=hook_tags)
+        result = resp.get_result()
+        if isinstance(result, dict):
+            self.update_field_cache(result)
+        gossip.trigger_with_tags('infinidat.sdk.post_object_deletion', {'obj': self, 'url': url}, tags=hook_tags)
+        return resp
 
 
 @gossip.register('infinidat.sdk.object_creation_failure')
