@@ -4,6 +4,7 @@ import dateutil
 import pkg_resources
 import click
 import logbook
+import logbook.more
 from infinisdk import Q
 from infinisdk.infinibox import InfiniBox
 from infinisdk.core.config import config
@@ -20,17 +21,20 @@ CUSTOMIZE_ENTRY_POINT = 'infinisdk.cli.customize'
 @click.group()
 @click.option("-v", "--verbose", count=True)
 @click.option("-q", "--quiet", count=True)
-@click.pass_context
-def cli(ctx, verbose, quiet):
-    console_handler = logbook.StderrHandler()
+@click.option("-l", "--log-file", type=click.Path())
+def cli(verbose, quiet, log_file):
+    console_handler = logbook.more.ColorizedStderrHandler()
     console_handler.level = min(max(logbook.TRACE, _DEFAULT_CONSOLE_LEVEL-verbose+quiet), logbook.CRITICAL)
-    ctx.obj['console_handler'] = console_handler
     console_handler.format_string = '{record.message}'
     console_handler.push_application()
+    if log_file:
+        file_handler = logbook.FileHandler(log_file, mode="w", bubble=True)
+        file_handler.push_application()
 
 
-def _get_system_object(system_name, should_login=False):
-    system = InfiniBox(system_name)
+def _get_system_object(system_name, port=None, should_login=False):
+    address = system_name if port is None else (system_name, port)
+    system = InfiniBox(address)
     has_auth = bool(system.api.get_auth())
     if not has_auth:
         msg = "Auth (username & password) wasn't set at {!r} file".format(config.root.ini_file_path)
@@ -59,8 +63,10 @@ def _interact(**local_vars):
 
 @cli.command()
 @click.option("-s", "--system-name", required=True)
-def interact(system_name):
-    system = _get_system_object(system_name)
+@click.option("-p", "--port", type=int)
+@click.option("--login", "should_login", is_flag=True, default=False)
+def interact(system_name, port, should_login):
+    system = _get_system_object(system_name, port=port, should_login=should_login)
     _interact(system=system)
 
 
@@ -68,7 +74,7 @@ def interact(system_name):
 def events():
     pass
 
-TIME_TEPMPLATE = 'YYYY-MM-DD HH:mm:ss'
+TIME_TEMPLATE = 'YYYY-MM-DD HH:mm:ss'
 
 def _convert_time_string_to_arrow(time_string, tzinfo):
     datetime_obj = dateutil.parser.parse(time_string)
@@ -116,7 +122,7 @@ def events_query(system_name, show_reporter, show_visibility, show_source_node_i
         event_time = event_info['timestamp']
         if display_in_local_time:
             event_time = event_time.to('local')
-        formatted = '{} {:5}'.format(event_time.format(TIME_TEPMPLATE), event_info['id'])
+        formatted = '{} {:5}'.format(event_time.format(TIME_TEMPLATE), event_info['id'])
         if show_reporter:
             formatted += ' {:10}'.format(event_info['reporter'])
         if show_visibility:
