@@ -1,13 +1,10 @@
+import http.client as httplib
 import operator
-
-from sentinels import NOTHING
-from vintage import deprecated
-
-from .._compat import httplib
 from ..core.config import config
+from sentinels import NOTHING
 
 
-class Feature(object):
+class Feature:
     def __init__(self, name, version, enabled):
         self.name = name
         self.version = version
@@ -18,7 +15,7 @@ def _get_predicate(opreator_func, value):
         return opreator_func(version, value)
     return predicate
 
-class Compatibility(object):
+class Compatibility:
 
     def __init__(self, system):
         self.system = system
@@ -56,7 +53,7 @@ class Compatibility(object):
         return getattr(self, 'has_{}'.format(feature_name))()
 
     def normalize_version_string(self, version):
-        return _InfiniboxVersion.parse(version)
+        return InfiniboxVersion.parse(version)
 
     def get_parsed_system_version(self):
         if self._system_version is None:
@@ -68,8 +65,8 @@ class Compatibility(object):
         return self.system.get_version().partition('.')[0]
 
     def get_version_as_float(self):
-        float_digit_list = self.system.get_version().split('.')[:2]
-        return float(".".join(float_digit_list))
+        version_tuple = self.get_parsed_system_version().version[:2]
+        return float(".".join(str(num) for num in version_tuple))
 
     def _init_features(self):
         resp = self.system.api.get("_features", assert_success=False)
@@ -90,20 +87,25 @@ class Compatibility(object):
             return default_version
         return feature.version
 
-    def has_feature(self, feature_key):
+    def _has_feature(self, feature_key):
         return self._get_feature_version(feature_key, NOTHING) is not NOTHING
 
+    def does_feature_exist(self, feature_key):
+        if self._features is None:
+            self._init_features()
+        return self._features.get(feature_key) is not None
+
     def has_npiv(self):
-        return self.has_feature('fc_soft_targets') or self.has_feature('fc/soft_targets')
+        return self._has_feature('fc_soft_targets') or self._has_feature('fc/soft_targets')
 
     def has_replication(self):
         return int(self.get_version_major()) >= 2
 
     def has_iscsi(self):
-        return self.has_feature('iscsi')
+        return self._has_feature('iscsi')
 
     def has_compression(self):
-        return self.has_feature('compression')
+        return self._has_feature('compression')
 
     def has_nas(self):
         return self._get_feature_version('nas', 0) >= 2
@@ -115,50 +117,106 @@ class Compatibility(object):
         return self._get_feature_version('metadata', 1)
 
     def has_consistency_groups(self):
-        return self.get_version_as_float() >= 2.2
+        return self.get_parsed_system_version() >= '2.2'
 
     def has_initiators(self):
-        return self.get_version_as_float() >= 2.2
+        return self.get_parsed_system_version() >= '2.2'
 
     def has_user_disabling(self):
         return self._get_feature_version("user_management", 0) >= 1
 
     def has_auth_sessions(self):
-        return self.has_feature('api_auth_sessions') or self.has_feature('api/auth_sessions')
+        return self._has_feature('api_auth_sessions') or self._has_feature('api/auth_sessions')
 
     def has_max_speed(self):
-        return self.get_version_as_float() > 2.2
+        return self.get_parsed_system_version() > '2.2'
 
     def has_writable_snapshots(self):
-        return self.has_feature('snapshots')
-
-    @deprecated('Use has_writable_snapshots instead', since='65.0')
-    def has_snapclones(self):
-        self.has_writable_snapshots()
+        return self._has_feature('snapshots')
 
     def has_sync_job_states(self):
-        return self.get_version_as_float() >= 3.0
+        return self.get_parsed_system_version() >= '3.0'
 
     def has_sync_replication(self):
-        return self.get_version_as_float() >= 4.0
+        return self.get_parsed_system_version() >= '4.0'
 
     def has_nas_replication(self):
-        return self.has_feature('filesystem_replicas')
+        return self._has_feature('filesystem_replicas')
 
     def has_qos(self):
-        return self.has_feature('qos')
+        return self._has_feature('qos')
 
     def has_treeq(self):
-        return self.has_feature('treeq')
+        return self._has_feature('treeq')
 
     def has_openvms(self):
-        return self.has_feature('open_vms')
+        return self._has_feature('open_vms')
+
+    def has_dot_snapshot(self):
+        return self._has_feature('dot_snapshots')
+
+    def has_tenants(self):
+        return self._has_feature('tenants')
+
+    def has_snapshot_lock(self):
+        return self.get_parsed_system_version() >= '4.0.30'
+
+    def has_auto_respawn(self):
+        return self.get_parsed_system_version() >= '4.0.30'
+
+    def has_active_active(self):
+        return self._has_feature('active_active')
+
+    def has_active_active_preferred_on_replica(self):
+        return self._get_feature_version("active_active", 0) > 0
+
+    def has_nlm(self):
+        return self._has_feature('nlm')
+
+    def has_local_users_auth(self):
+        return self._has_feature('local_users_auth')
+
+    def has_fips(self):
+        return self._has_feature('fips')
+
+    def has_replica_auto_create(self):
+        return self._has_feature('replica_auto_create')
+
+    def has_active_active_cg(self):
+        return self._get_feature_version("active_active", 0) > 1
+
+    def has_events_db(self):
+        return self._has_feature('events_db')
+
+    def has_concurrent_replication(self):
+        return self._get_feature_version("active_active", 0) > 2
+
+    def has_kms(self):
+        return  self._has_feature('kms')
+
+    def has_event_retention(self):
+        return self._has_feature('event_retention')
+
+    def has_vvol(self):
+        return self._has_feature('vvol')
+
+    def has_sg_delete_limit(self):
+        return self.get_parsed_system_version() >= '6.0.0'
+
+    def has_active_active_suspend(self):
+        return self._get_feature_version("active_active", 0) > 3
+
+    def has_platform(self):
+        return self.get_parsed_system_version() >= '5.5.10'
+
+    def has_ism(self):
+        return self.has_fips()
 
 
 _VERSION_TUPLE_LEN = 5
 
 
-class _InfiniboxVersion(object):
+class InfiniboxVersion:
 
     def __init__(self, version_tuple, is_dev, is_odd=False):
         self.version = version_tuple
@@ -167,7 +225,7 @@ class _InfiniboxVersion(object):
 
     @classmethod
     def parse(cls, version):
-        if isinstance(version, _InfiniboxVersion):
+        if isinstance(version, InfiniboxVersion):
             return version
         before_dash, _, after_dash = version.partition('-')
         is_dev = is_odd = False
@@ -225,6 +283,6 @@ class _InfiniboxVersion(object):
         extra_info = ""
         if self._is_dev:
             extra_info += " dev version"
-        if not self._is_odd_version:
+        if self._is_odd_version:
             extra_info += " (unknown structure)"
         return "<InfiniboxVersion: {}{}>".format(self.version, extra_info)
