@@ -1,7 +1,8 @@
+import functools
 import operator
 from numbers import Number
 
-from .._compat import cmp, iteritems, itervalues, sorted  # pylint: disable=redefined-builtin
+from ..core.utils.python import cmp
 from ..core.object_query import QueryBase
 
 
@@ -49,9 +50,11 @@ class InfiniBoxComponentQuery(ComponentQueryBase):
         super(InfiniBoxComponentQuery, self).__init__(system, object_type.get_plural_name(), *predicates, **kw)
         self.object_type = object_type
         self.sort_criteria = tuple()
-        self._force_fetch = bool(self.predicates) or \
-            any(self.object_type.fields.get_or_fabricate(field_name).cached != True for field_name in self.kw)
-
+        # predicates' field attribute is QField (non the object field), therefore, we should get the object's one
+        # for checking its cached attribute
+        field_names = [pred.field.name for pred in predicates] + list(kw)
+        relevant_fields = [self.object_type.fields.get_or_fabricate(field_name) for field_name in field_names]
+        self._force_fetch = any(field.cached is not True for field in relevant_fields)
 
     def _get_items(self):
         returned = self._fetched_items
@@ -68,11 +71,11 @@ class InfiniBoxComponentQuery(ComponentQueryBase):
                 return 0
 
             with self._get_binder().fetch_tree_once_context(force_fetch=self._force_fetch, with_logging=False):
-                all_components = itervalues(self.system.components._components_by_id)  # pylint: disable=protected-access
+                all_components = self.system.components._components_by_id.values()  # pylint: disable=protected-access
                 returned = [item for item in all_components if self.passed_filtering(item)]
 
             if self.sort_criteria:
-                returned = sorted(returned, cmp=_sort_cmp_items)
+                returned = sorted(returned, key=functools.cmp_to_key(_sort_cmp_items))
             self._fetched_items = returned
         return returned
 
@@ -84,6 +87,8 @@ class InfiniBoxComponentQuery(ComponentQueryBase):
     def passed_filtering(self, item):
         if self.object_type != self.system.components.object_type and self.object_type != type(item):
             return False
+        if not item.is_in_system():
+            return False
         for predicate in self.predicates:
             try:
                 op_func = getattr(operator, predicate.operator_name)
@@ -92,7 +97,7 @@ class InfiniBoxComponentQuery(ComponentQueryBase):
             item_value = item.get_field(predicate.field.name)
             if not op_func(item_value, predicate.value):
                 return False
-        for k, v in iteritems(self.kw):
+        for k, v in self.kw.items():
             if item.get_field(k) != v:
                 return False
         return True
@@ -123,6 +128,8 @@ class InfiniBoxGenericComponentQuery(ComponentQueryBase):
         with self.system.components.fetch_tree_once_context(force_fetch=self._force_fetch, with_logging=False):
             fields = set(predicate.field.name for predicate in self.predicates) | set(self.kw)
             for component_type in self.system.components.get_component_types():
+                if not component_type.is_supported(self.system):
+                    continue
                 if not fields.issubset(set(field.name for field in component_type.fields)):
                     continue
                 assert component_type
