@@ -1,19 +1,20 @@
 import copy
+import functools
+import uuid
 
-from .._compat import ExitStack, zip  # pylint: disable=redefined-builtin
 from ..core.field import Field
 from ..core.field_filter import FieldFilter
 from ..core.system_component import SystemComponentsBinder
 from ..core.system_object import BaseSystemObject
 from ..core.exceptions import ObjectNotFound
 from ..core.type_binder import MonomorphicBinder
-from ..core.translators_and_types import WWNType, CapacityType
+from ..core.translators_and_types import WWNType, CapacityType, MunchType, MunchListType
 from mitba import cached_method
 from .component_query import InfiniBoxComponentQuery, InfiniBoxGenericComponentQuery
 from ..core.bindings import InfiniSDKBinding, ListOfRelatedComponentBinding, RelatedComponentBinding
 
 from collections import defaultdict
-from contextlib import contextmanager
+from contextlib import contextmanager, ExitStack
 from logbook import Logger
 from urlobject import URLObject as URL
 from vintage import deprecated, warn_deprecation
@@ -22,35 +23,39 @@ from vintage import deprecated, warn_deprecation
 _logger = Logger(__name__)
 _UID_DEPRECATION_MSG = "Direct usage of the 'id' field is deprecated. Use 'uid' field instead"
 
+
 def _normalize_id_predicate(predicate, component_type):
     if predicate.field.name == 'id' and isinstance(predicate.value, str):
         warn_deprecation(_UID_DEPRECATION_MSG)
         return FieldFilter(component_type.fields.uid, predicate.operator_name, predicate.value)
     return predicate
 
+
 class InfiniBoxSystemComponents(SystemComponentsBinder):
 
     def __init__(self, system):
         super(InfiniBoxSystemComponents, self).__init__(InfiniBoxSystemComponent, system)
-        self._initialize()
-
-    def _initialize(self):
         self.system_component = System(self.system, {'parent_id': "", 'id': 0})
         self.cache_component(self.system_component)
         RackType = self.racks.object_type
         self._rack_1 = RackType(self.system, {'parent_id': self.system_component.id, 'rack': 1})
         self.cache_component(self._rack_1)
+        self._initialize()
+
+    def _initialize(self):
         self._fetched_nodes = False
         self._fetched_others = False
         self._fetched_service_clusters = False
-        self._deps_by_compoents_tree = defaultdict(set)
+        self._fetched_external_clusters = False
+        self._deps_by_components_tree = defaultdict(set)
+        self._initialization_uuid = uuid.uuid4()
 
     def invalidate_cache(self):
         super(InfiniBoxSystemComponents, self).invalidate_cache()
         self._initialize()
 
     def get_depended_components_type(self, component_type):
-        deps = self._deps_by_compoents_tree[component_type].copy()
+        deps = self._deps_by_components_tree[component_type].copy()
         for dep_type in deps.copy():
             deps.update(self.get_depended_components_type(dep_type))
         return deps
@@ -64,6 +69,9 @@ class InfiniBoxSystemComponents(SystemComponentsBinder):
     def should_fetch_service_clusters(self):
         return not self._fetched_service_clusters
 
+    def should_fetch_external_clusters(self):
+        return not self._fetched_external_clusters
+
     def mark_fetched_nodes(self):
         self._fetched_nodes = True
 
@@ -74,14 +82,17 @@ class InfiniBoxSystemComponents(SystemComponentsBinder):
     def mark_fetched_service_clusters(self):
         self._fetched_service_clusters = True
 
+    def mark_fetched_external_clusters(self):
+        self._fetched_external_clusters = True
+
     def get_rack_1(self):
         return self._rack_1
 
-    def find(self, component_type=None, *predicates, **kw): # pylint: disable=arguments-differ
-        # component_type is the name of the component
-        if component_type is None:
+    def find(self, *predicates, **kw):
+        component_name = kw.pop('component_type', None)
+        if component_name is None:
             return InfiniBoxGenericComponentQuery(self.system, *predicates, **kw)
-        component_type = self._COMPONENTS_BY_TYPE_NAME[component_type]
+        component_type = self._COMPONENTS_BY_TYPE_NAME[component_name]  # pylint: disable=unsubscriptable-object
         component_collection = self.system.components[component_type]
         return component_collection.find(*predicates, **kw)
 
@@ -118,6 +129,9 @@ class InfiniBoxComponentBinder(MonomorphicBinder):
     def should_force_fetching_from_cache(self):
         return self._force_fetching_from_cache
 
+    def get_by_uid(self, uid):
+        return self.get(uid=uid)
+
     def get_by_id_lazy(self, id):  # pylint: disable=redefined-builtin
         returned = self.safe_get_by_id(id)
         if returned is None:
@@ -183,6 +197,9 @@ class InfiniBoxComponentBinder(MonomorphicBinder):
         elif self.object_type is components.service_clusters.object_type:
             if force_fetch or components.should_fetch_service_clusters():
                 self._fetch_service_clusters()
+        elif self.object_type is components.external_clusters.object_type:
+            if force_fetch or components.should_fetch_external_clusters():
+                self._fetch_external_clusters()
         else:
             if force_fetch or components.should_fetch_all():
                 rack_1 = components.get_rack_1()
@@ -197,6 +214,15 @@ class InfiniBoxComponentBinder(MonomorphicBinder):
         for cluster_data in clusters_data:
             service_cluster_type.construct(self.system, cluster_data, None)
 
+    def _fetch_external_clusters(self):
+        components = self.system.components
+        service_cluster_type = components.external_clusters.object_type
+        url = service_cluster_type.get_url_path(self.system)
+        clusters_data = self.system.api.get(url).get_result()
+        components.mark_fetched_external_clusters()
+        for cluster_data in clusters_data:
+            service_cluster_type.construct(self.system, cluster_data, None)
+
 
 class InfiniBoxSystemComponent(BaseSystemObject):
     BINDER_CLASS = InfiniBoxComponentBinder
@@ -211,9 +237,22 @@ class InfiniBoxSystemComponent(BaseSystemObject):
     def get_id(self):
         return self.get_uid()
 
+    def is_in_system(self):
+        return self._construct_uuid == self.system.components._initialization_uuid  # pylint: disable=protected-access
+
     def __deepcopy__(self, memo):
         return self.construct(self.system, copy.deepcopy(self._cache, memo), self.get_parent_id())
 
+    def invalidate_cache(self, *field_names):
+        non_invalidated_fields = set(name for field in self.fields for name in (field.name, field.api_name)
+                                     if field.is_identity)
+        field_names_to_invalidate = set(field_names or self._cache.keys()) - set(non_invalidated_fields)
+        if field_names_to_invalidate:
+            super(InfiniBoxSystemComponent, self).invalidate_cache(*field_names_to_invalidate)
+        else:
+            assert not field_names, \
+                "Cannot invalidate only these fields: {}".format(', '.join(non_invalidated_fields))
+
     def _deduce_from_cache(self, field_names, from_cache):
         collection = self.system.components[self.get_plural_name()]
         if collection.should_force_fetching_from_cache():
@@ -238,16 +277,23 @@ class InfiniBoxSystemComponent(BaseSystemObject):
         return self.system.components[self.get_plural_name()]
     get_collection = get_binder
 
+    @classmethod
+    def _iter_sub_component_fields(cls, system):
+        for field in cls.fields:
+            if system.is_field_supported(field) and isinstance(field.binding, ListOfRelatedComponentBinding):
+                yield field
+
+
     def get_sub_components(self):
-        for field in self.fields:
-            if isinstance(field.binding, ListOfRelatedComponentBinding):
-                for component in self.get_field(field.name):
-                    yield component
+        for field in self._iter_sub_component_fields(self.system):
+            for component in self.get_field(field.name):
+                yield component
 
     def refresh_cache(self):
         data = self.system.api.get(self.get_this_url_path()).get_result()
         self.construct(self.system, data, self.get_parent_id())
 
+
     @classmethod
     def construct(cls, system, data, parent_id, allow_partial_fields=False):    # pylint: disable=arguments-differ
         # pylint: disable=protected-access
@@ -258,17 +304,17 @@ class InfiniBoxSystemComponent(BaseSystemObject):
             component_type = cls.get_type_name()
             object_type = system.components._COMPONENTS_BY_TYPE_NAME.get(component_type, InfiniBoxSystemComponent)
             returned = object_type(system, data)
-            system.components._deps_by_compoents_tree[type(returned.get_parent())].add(object_type)
+            system.components._deps_by_components_tree[type(returned.get_parent())].add(object_type)
             system.components.cache_component(returned)
         else:
             returned.update_field_cache(data)
-        for field in cls.fields:
-            if isinstance(field.binding, ListOfRelatedComponentBinding):
-                try:
-                    field.binding.get_value_from_api_object(system, cls, returned, data)
-                except KeyError:
-                    if not allow_partial_fields:
-                        raise
+        for field in cls._iter_sub_component_fields(system):
+            try:
+                field.binding.get_value_from_api_object(system, cls, returned, data)
+            except KeyError:
+                if not allow_partial_fields:
+                    raise
+        returned._construct_uuid = system.components._initialization_uuid
         return returned
 
 
@@ -276,10 +322,11 @@ class InfiniBoxSystemComponent(BaseSystemObject):
 @InfiniBoxSystemComponents.install_component_type
 class Rack(InfiniBoxSystemComponent):
     FIELDS = [
-        Field("index", api_name="rack", type=int, cached=True),
+        Field("index", api_name="rack", type=int, is_identity=True),
         Field("enclosures", type=list, binding=ListOfRelatedComponentBinding()),
         Field("nodes", type=list, binding=ListOfRelatedComponentBinding()),
         Field("bbus", api_name='ups', type=list, binding=ListOfRelatedComponentBinding()),
+        Field("pdus", type=list, binding=ListOfRelatedComponentBinding()),
     ]
 
     @classmethod
@@ -294,13 +341,17 @@ class Rack(InfiniBoxSystemComponent):
         return self.get_specific_rack_url(self.get_index())
 
     def refresh_without_enclosures(self):
-        fields = ",".join(field.api_name for field in self.fields if field.name != 'enclosures')
+        fields = ",".join(field.api_name for field in self.fields
+                          if field.name not in ('enclosures', 'uid', 'parent_id'))
         url = self.get_this_url_path().add_query_param('fields', fields)
         data = self.system.api.get(url).get_result()
         self.system.components.mark_fetched_nodes()
         data['enclosures'] = []
         self.construct(self.system, data, self.get_parent_id())
 
+    def is_in_system(self):
+        return True
+
     def refresh_cache(self):
         self.system.components.mark_fetched_all()
         super(Rack, self).refresh_cache()
@@ -310,7 +361,7 @@ class Rack(InfiniBoxSystemComponent):
 class Enclosure(InfiniBoxSystemComponent):
     FIELDS = [
         Field("api_id", api_name="id", type=int, cached=True),
-        Field("index", api_name="id", type=int, cached=True),
+        Field("index", api_name="id", type=int, is_identity=True),
         Field("drives", type=list, binding=ListOfRelatedComponentBinding()),
         Field("state", cached=False),
     ]
@@ -332,6 +383,12 @@ class Nodes(InfiniBoxComponentBinder):
                     if ip.ip_address == ip_address:
                         return self.system.network_interfaces.get_by_id_lazy(ip.interface_id).get_node()
 
+    def get_by_mgmt_ip(self, ip_address):
+        with self.system.components.eth_ports.fetch_once_context():
+            for port in self.system.components.eth_ports:
+                if port.get_role() == 'MANAGEMENT' and port.get_ip_v4_addr() == ip_address:
+                    return port.get_node()
+
     def refresh_fields(self, field_names):
         assert isinstance(field_names, (list, tuple)), "field_names must be either a list or a tuple"
         field_names_str = ",".join(set(field_names).union(['id']))
@@ -346,7 +403,7 @@ class Node(InfiniBoxSystemComponent):
     BINDER_CLASS = Nodes
     FIELDS = [
         Field("api_id", api_name="id", type=int, cached=True),
-        Field("index", api_name="id", type=int, cached=True),
+        Field("index", api_name="id", type=int, is_identity=True),
         Field("name", cached=True),
         Field("model", cached=True),
         Field("ib_ports", type=list, binding=ListOfRelatedComponentBinding()),
@@ -355,16 +412,18 @@ class Node(InfiniBoxSystemComponent):
         Field("drives", type=list, binding=ListOfRelatedComponentBinding("local_drives")),
         Field("services", type=list, binding=ListOfRelatedComponentBinding()),
         Field("state", cached=False),
+        Field("security", cached=False, type=MunchType, feature_name='fips'),
+        Field("tpm", cached=False, type=MunchType, feature_name='fips'),
     ]
 
-    def is_active(self):
-        return self.get_state() == 'ACTIVE'
+    def is_active(self, **kwargs):
+        return self.get_state(**kwargs) == 'ACTIVE'
 
-    def is_degraded(self):
-        return self.get_state() == 'DEGRADED'
+    def is_degraded(self, **kwargs):
+        return self.get_state(**kwargs) == 'DEGRADED'
 
-    def is_failed(self):
-        return self.get_state() == 'FAILED'
+    def is_failed(self, **kwargs):
+        return self.get_state(**kwargs) == 'FAILED'
 
     @classmethod
     def get_url_path(cls, system):
@@ -387,14 +446,11 @@ class Node(InfiniBoxSystemComponent):
         """
         return self.get_service('core')
 
-    def _get_tags(self):
-        return ['infinibox', 'node{}'.format(self.get_index())]
-
 
 @InfiniBoxSystemComponents.install_component_type
 class LocalDrive(InfiniBoxSystemComponent):
     FIELDS = [
-        Field("index", api_name="drive_index", type=int, cached=True),
+        Field("index", api_name="drive_index", type=int, is_identity=True),
         Field("model"),
         Field("vendor"),
         Field("firmware"),
@@ -402,6 +458,7 @@ class LocalDrive(InfiniBoxSystemComponent):
         Field("type"),
         Field("serial_number"),
         Field("node", api_name="node_index", type=int, cached=True, binding=RelatedComponentBinding()),
+        Field("encryption_state", type=bool, feature_name="fips"),
     ]
 
     @classmethod
@@ -414,11 +471,11 @@ class LocalDrive(InfiniBoxSystemComponent):
         this_url = parent_url.add_path("drives").add_path(str(self.get_index()))
         return this_url
 
-    def is_active(self):
-        return self.get_state() == 'OK'
+    def is_active(self, **kwargs):
+        return self.get_state(**kwargs) == 'OK'
 
-    def is_ssd(self):
-        return self.get_type() == 'SSD'
+    def is_ssd(self, **kwargs):
+        return self.get_type(**kwargs) == 'SSD'
 
 
 @InfiniBoxSystemComponents.install_component_type
@@ -430,7 +487,7 @@ class EthPort(InfiniBoxSystemComponent):
         Field("max_speed", type=int, feature_name="max_speed"),
         Field("device_name", api_name="name"),
         Field("port_number", type=int, cached=True),
-        Field("index", api_name="id", type=int, cached=True),
+        Field("index", api_name="id", type=int, is_identity=True),
         Field("node", api_name="node_index", type=int, cached=True, binding=RelatedComponentBinding()),
         Field("role", cached=True),
         Field("name", cached=True),
@@ -448,17 +505,17 @@ class EthPort(InfiniBoxSystemComponent):
     def get_type_name(cls):
         return "eth_port"
 
-    def is_link_up(self):
-        if self.get_state() != 'OK':
+    def is_link_up(self, **kwargs):
+        if self.get_state(**kwargs) != 'OK':
             return False
-        return self.get_link_state().lower() in ("link up", "up")
+        return self.get_link_state(**kwargs).lower() in ("link up", "up")
 
 
 @InfiniBoxSystemComponents.install_component_type
 class IbPort(InfiniBoxSystemComponent):
     FIELDS = [
         Field("api_id", api_name="id", type=int, cached=True),
-        Field("index", api_name="id", type=int, cached=True),
+        Field("index", api_name="id", type=int, is_identity=True),
         Field("firmware"),
         Field("last_probe_timestamp", type=int),
         Field("link_state", cached=False, new_to="3.0"),
@@ -475,12 +532,14 @@ class IbPort(InfiniBoxSystemComponent):
     def get_type_name(cls):
         return "ib_port"
 
-    def is_link_up(self):
-        if self.get_state() != 'OK':
+    def is_link_up(self, *, from_cache=False):
+        if not from_cache:
+            self.refresh_cache()
+        if self.get_state(from_cache=True) != 'OK':
             return False
         if not self.is_field_supported('link_state'):
             return True
-        link_state = self.get_link_state()
+        link_state = self.get_link_state(from_cache=True)
         return link_state and link_state.lower() == "up"
 
 
@@ -500,7 +559,7 @@ class FcPort(InfiniBoxSystemComponent):
     BINDER_CLASS = FcPorts
     FIELDS = [
         Field("api_id", api_name="id", type=int, cached=True),
-        Field("index", api_name="id", type=int, cached=True),
+        Field("index", api_name="id", type=int, is_identity=True),
         Field("wwpn", is_identity=True, cached=True, type=WWNType),
         Field("node", api_name="node_index", type=int, cached=True, binding=RelatedComponentBinding()),
         Field("state", cached=False),
@@ -511,42 +570,53 @@ class FcPort(InfiniBoxSystemComponent):
         Field("enabled", type=bool),
     ]
 
-    def is_link_up(self):
-        if not self.is_enabled():
+    def is_link_up(self, **kwargs):
+        if not self.is_enabled(**kwargs):
             return False
-        if self.get_state() != 'OK':
+        if self.get_state(**kwargs) != 'OK':
             return False
-        return self.get_link_state().lower() in ("link up", "up")
+        return self.get_link_state(**kwargs).lower() in ("link up", "up")
 
     @classmethod
     def get_type_name(cls):
         return "fc_port"
 
-    def is_hard_port(self):
+    def is_hard_port(self, **kwargs):
         if self.system.compat.has_npiv():
-            return self.get_role() == 'HARD_PORT'
+            return self.get_role(**kwargs) == 'HARD_PORT'
         return True
 
-    def is_soft_port(self):
+    def is_soft_port(self, **kwargs):
         if self.system.compat.has_npiv():
-            return self.get_role() == 'SOFT_PORT'
+            return self.get_role(**kwargs) == 'SOFT_PORT'
         return False
 
-    def get_target_addresses(self):
-        if self.is_soft_port():
-            return self.get_soft_target_addresses()
-        return set([self.get_wwpn()])
+    def get_target_addresses(self, **kwargs):
+        if self.is_soft_port(**kwargs):
+            return self.get_soft_target_addresses(**kwargs)
+        return set([self.get_wwpn(**kwargs)])
+
+    def enable(self, role):
+        """Enables the FC port with the specified role (SOFT_PORT/HARD_PORT)"""
+        self.system.api.post(self.get_this_url_path().add_path('enable'), data={'role': role})
+        self.invalidate_cache('enabled')
+
+    def disable(self):
+        """Disables the FC Port"""
+        self.system.api.post(self.get_this_url_path().add_path('disable'), data={})
+        self.invalidate_cache('enabled')
 
 
 @InfiniBoxSystemComponents.install_component_type
 class Drive(InfiniBoxSystemComponent):
     FIELDS = [
-        Field("index", api_name="drive_index", type=int, is_identity=True, cached=True),
+        Field("index", api_name="drive_index", type=int, is_identity=True),
         Field("enclosure_index", type=int, cached=True),
         Field("enclosure", api_name="enclosure_index", type=int, cached=True, binding=RelatedComponentBinding()),
         Field("serial_number"),
         Field("capacity", api_name="bytes_capacity", type=CapacityType),
         Field("state", cached=False),
+        Field("encryption_state", type=bool, feature_name="fips"),
     ]
 
     def get_paths(self, from_cache=False):
@@ -555,14 +625,14 @@ class Drive(InfiniBoxSystemComponent):
                 for node, can_access_node in zip(self.system.components.nodes, node_access_flags)
                 if can_access_node]
 
-    def is_active(self):
-        return self.get_state() == 'ACTIVE'
+    def is_active(self, **kwargs):
+        return self.get_state(**kwargs) == 'ACTIVE'
 
 
 @InfiniBoxSystemComponents.install_component_type
 class Service(InfiniBoxSystemComponent):
     FIELDS = [
-        Field("index", api_name="name", cached=True),
+        Field("index", api_name="name", is_identity=True),
         Field("name", is_identity=True, cached=True),
         Field("role", cached=False),
         Field("state", cached=False),
@@ -574,21 +644,21 @@ class Service(InfiniBoxSystemComponent):
         except ObjectNotFound:
             raise NotImplementedError("This service ({}) doesn't support CLM".format(self.get_name()))
 
-    def is_active(self):
-        return self.get_state() == 'ACTIVE'
+    def is_active(self, **kwargs):
+        return self.get_state(**kwargs) == 'ACTIVE'
 
-    def is_inactive(self):
+    def is_inactive(self, **kwargs):
         # Workaround for INFINIBOX-18309 & INFINIBOX-18308 & INFINIBOX-18647 & INFINIBOX-17256
-        return self.get_state() in ('INACTIVE', 'PROCESS_FINISHED', 'SHUTDOWN', 'INVALID')
+        return self.get_state(**kwargs) in ('INACTIVE', 'PROCESS_FINISHED', 'SHUTDOWN', 'INVALID')
 
-    def is_master(self):
-        return self.get_role() == 'MASTER'
+    def is_master(self, **kwargs):
+        return self.get_role(**kwargs) == 'MASTER'
 
-    def is_secondary(self):
-        return self.get_role() == 'SECONDARY'
+    def is_secondary(self, **kwargs):
+        return self.get_role(**kwargs) == 'SECONDARY'
 
-    def is_member(self):
-        return self.get_role() == 'MEMBER'
+    def is_member(self, **kwargs):
+        return self.get_role(**kwargs) == 'MEMBER'
 
     def get_node(self):
         return self.get_parent()
@@ -597,7 +667,7 @@ class Service(InfiniBoxSystemComponent):
 @InfiniBoxSystemComponents.install_component_type
 class ServiceCluster(InfiniBoxSystemComponent):
     FIELDS = [
-        Field("index", api_name="name", cached=True),
+        Field("index", api_name="name", is_identity=True),
         Field("name", is_identity=True, cached=True),
         Field("state", api_name="cluster_state", cached=False),
         ]
@@ -620,18 +690,69 @@ class ServiceCluster(InfiniBoxSystemComponent):
         return [self.system.components.nodes.get(index=service_info['node_id']).get_service(self.get_name())
                 for service_info in self.get_field('node_states')]
 
-    def is_active(self):
-        return self.get_state() == 'ACTIVE'
+    def is_active(self, **kwargs):
+        return self.get_state(**kwargs) == 'ACTIVE'
 
-    def is_inactive(self):
-        return self.get_state() in ('INACTIVE', 'SHUTDOWN')
+    def is_inactive(self, **kwargs):
+        return self.get_state(**kwargs) in ('INACTIVE', 'SHUTDOWN')
+
+    def is_degraded(self, **kwargs):
+        return self.get_state(**kwargs) == 'DEGRADED'
+
+
+def _ensure_supported_external_cluster(func):
+    @functools.wraps(func)
+    def inner(self, *args, **kwargs):
+        if self.get_name() not in ("elastic", "postgresql"):
+            raise NotImplementedError("The getter for cluster {} is not supported".format(self.get_name()))
+        return func(self, *args, **kwargs)
+    return inner
+
+@InfiniBoxSystemComponents.install_component_type
+class ExternalServiceCluster(InfiniBoxSystemComponent):
+    FIELDS = [
+        Field("index", api_name="name", is_identity=True),
+        Field("name", is_identity=True, cached=True),
+        Field("state", api_name="cluster_state", cached=False),
+        Field("node_states", type=list),
+        ]
+
+    @classmethod
+    def get_type_name(cls):
+        return "external_cluster"
+
+    @classmethod
+    def get_url_path(cls, system):
+        return URL('external_services')
+
+    @cached_method
+    def get_this_url_path(self):
+        services_url = self.get_url_path(self.system)
+        this_url = services_url.add_path(str(self.get_index()))
+        return this_url
+
+    @classmethod
+    def is_supported(cls, system): # pylint: disable=unused-argument
+        return system.compat.has_events_db()
+
+    @_ensure_supported_external_cluster
+    def is_steady(self, **kwargs):
+        return self.get_field("health", **kwargs)["initializing_shards"] == 0 if self.get_name() == "elastic" else True
+
+    @_ensure_supported_external_cluster
+    def is_active(self, **kwargs):
+        return self.get_state(**kwargs) == 'GREEN'
+
+    @_ensure_supported_external_cluster
+    def is_degraded(self, **kwargs):
+        return self.get_state(**kwargs) == 'YELLOW'
 
 
 @InfiniBoxSystemComponents.install_component_type
 class BBU(InfiniBoxSystemComponent):
     FIELDS = [
         Field("api_id", api_name="id", type=int, cached=True),
-        Field("index", api_name="id", type=int, cached=True),
+        Field("index", api_name="id", type=int, is_identity=True),
         Field("state", cached=False),
         Field("on_battery", api_name="onBattery", type=bool, cached=False),
         Field("charging", type=bool, cached=False),
@@ -646,35 +767,64 @@ class BBU(InfiniBoxSystemComponent):
         return self.get_url_path(self.system).add_path(str(self.get_index()))
 
 
+@InfiniBoxSystemComponents.install_component_type
+class PDU(InfiniBoxSystemComponent):
+    FIELDS = [
+        Field("index", api_name="id", type=int, is_identity=True),
+        Field("api_id", api_name="id", type=int, is_identity=True),
+        Field("model"),
+        Field("vendor"),
+        Field("firmware"),
+        Field("state", cached=False),
+        Field("state_description"),
+        Field("power_ports", type=MunchListType),
+        Field("power_consumption", type=int),
+        Field("probe_ttl", type=int),
+        Field("last_probe_timestamp", type=int),
+    ]
+
+    @classmethod
+    def get_url_path(cls, system):
+        return cls.BASE_URL.add_path(cls.get_plural_name())
+
+    @cached_method
+    def get_this_url_path(self):
+        return self.get_url_path(self.system).add_path(str(self.get_index()))
+
+
 @InfiniBoxSystemComponents.install_component_type
 class System(InfiniBoxSystemComponent):
     FIELDS = [
-        Field("api_id", api_name="id", type=int, cached=True),
-        Field("index", api_name="id", type=int, cached=True),
+        Field("api_id", api_name="id", type=int, cached=True, is_identity=True),
+        Field("index", api_name="id", type=int, is_identity=True),
         Field("operational_state", type=dict, cached=False),
+        Field("security", type=dict, feature_name='fips'),
     ]
 
     @cached_method
     def get_this_url_path(self):
         return URL('system')
 
+    def is_in_system(self):
+        return True
+
     def get_state(self, *args, **kwargs):
         return self.get_operational_state(*args, **kwargs)['state']
 
-    def safe_get_state(self):
+    def safe_get_state(self, **kwargs):
         try:
-            return self.get_state()
+            return self.get_state(**kwargs)
         except Exception:  # pylint: disable=broad-except
             return None
 
-    def is_active(self):
-        return self.safe_get_state() == 'ACTIVE'
+    def is_active(self, **kwargs):
+        return self.safe_get_state(**kwargs) == 'ACTIVE'
 
-    def is_stand_by(self):
-        return self.safe_get_state() == 'STANDBY'
+    def is_stand_by(self, **kwargs):
+        return self.safe_get_state(**kwargs) == 'STANDBY'
 
-    def is_down(self):
-        return self.safe_get_state() is None
+    def is_down(self, **kwargs):
+        return self.safe_get_state(**kwargs) is None
 
     def refresh_cache(self):
         data = self.system.api.get(self.get_this_url_path()).get_result()
