@@ -1,14 +1,12 @@
 # pylint: disable=no-member
 import logbook
 import gossip
-import gadget
 import functools
-
 from ..core.utils import end_reraise_context
 from ..core import Field, MillisecondsDatetimeType
 from ..core.api.special_values import OMIT
-from ..core.bindings import RelatedObjectNamedBinding
-from ..core.exceptions import CannotGetReplicaState, InvalidUsageException, TooManyObjectsFound, UnknownSystem
+from ..core.bindings import RelatedObjectNamedBinding, ReplicaEntityBinding
+from ..core.exceptions import CannotGetReplicaState, InvalidUsageException, TooManyObjectsFound
 from ..core.translators_and_types import MillisecondsDeltaType, CapacityType
 from ..core.type_binder import TypeBinder
 from ..core.system_object import SystemObject
@@ -20,7 +18,7 @@ def require_sync_replication(func):
     def new_func(self, *args, **kwargs):
         if self.system.compat.has_sync_replication():
             return func(self, *args, **kwargs)
-        raise NotImplementedError("not availble in this version")
+        raise NotImplementedError("not available in this version")
     return new_func
 
 
@@ -57,12 +55,18 @@ class ReplicaBinder(TypeBinder):
             return self.replicate_entity_create_target(entity, link, remote_pool=remote_pool, **kw)
         return self.replicate_entity_existing_target(entity, link, remote_entity=remote_entity, **kw)
 
-    def replicate_entity_create_target(self, entity, link, remote_pool=OMIT, **kw):
+    def replicate_entity_create_target(self, entity, link, remote_pool=OMIT, remote_entity_names=OMIT, **kw):
         """Replicates an entity, creating its remote replica on the specified pool
 
         :param remote_pool: Remote pool to use for entity creation on the remote side
+        :param remote_entity_names: A list or tuple containing the entity names created on remote side.
+                                    In case of cg - controls the entity names, not the cg name.
         """
-        return self.system.replicas.create(link=link, entity_pairs=self._build_entity_pairs_create_target(entity),
+        if remote_entity_names is not OMIT:
+            assert isinstance(remote_entity_names, (list, tuple)), "object remote_entity_names must be tuple or list"
+        return self.system.replicas.create(link=link,
+                                           entity_pairs=self._build_entity_pairs_create_target(remote_entity_names,
+                                                                                               entity),
                                            remote_pool_id=remote_pool.id if remote_pool is not OMIT else OMIT,
                                            **self._get_extra_replica_kwargs(kw, entity))
 
@@ -78,7 +82,7 @@ class ReplicaBinder(TypeBinder):
                                            **self._get_extra_replica_kwargs(kw, entity, remote_entity))
 
     def replicate_entity_use_base(self, entity, link, local_snapshot, remote_snapshot, member_mappings=None, **kw):
-        """Replicates an entity, using an existing remote entity and a base snapthot on both sides
+        """Replicates an entity, using an existing remote entity and a base snapshot on both sides
 
         :param local_snapshot: Local base snapshot to use
         :param remote_snapshot: Remote base snapshot to use
@@ -106,6 +110,8 @@ class ReplicaBinder(TypeBinder):
 
     def _get_extra_replica_kwargs(self, kw, entity, remote_entity=None):
         returned = kw
+        if 'base_action' in returned:
+            raise InvalidUsageException("replicate_entity() doesn't support creating replica with base_action format")
         assert 'entity_type' not in returned
         assert 'local_cg_id' not in returned
         assert 'remote_cg_id' not in returned
@@ -126,14 +132,19 @@ class ReplicaBinder(TypeBinder):
             return parent.id
         return entity.id
 
-    def _build_entity_pairs_create_target(self, entity):
+    def _build_entity_pairs_create_target(self, remote_entity_names, entity):
         returned = []
+        names_index = 0
         for sub_entity in self._get_sub_entities(entity):
-            returned.append({
+            new_pair = {
                 'remote_base_action': 'CREATE',
                 'local_entity_id': sub_entity.id,
                 'remote_entity_id': None,
-                })
+                }
+            if remote_entity_names is not OMIT and names_index < len(remote_entity_names):
+                new_pair['remote_entity_name'] = remote_entity_names[names_index]
+                names_index += 1
+            returned.append(new_pair)
         return returned
 
     def _build_entity_pairs_existing(self, local_entity, remote_entity, member_mappings, use_snapshots,
@@ -186,69 +197,83 @@ class Replica(SystemObject):
     FIELDS = [
 
         Field('id', type=int, is_identity=True, is_filterable=True),
-        Field('description', is_filterable=True, mutable=True),
+        Field('description', is_filterable=True, is_sortable=True, mutable=True),
         Field('updated_at', type=MillisecondsDatetimeType, is_filterable=True, is_sortable=True),
-        Field('created_at', type=MillisecondsDatetimeType, is_sortable=True, is_filterable=True, cached=True),
+        Field('created_at', type=MillisecondsDatetimeType, is_filterable=True, is_sortable=True, cached=True),
         Field('link', api_name='link_id', binding=RelatedObjectNamedBinding('links'),
               type='infinisdk.infinibox.link:Link', creation_parameter=True),
-        Field('entity_pairs', type=list, creation_parameter=True),
+        Field('entity_pairs', type=list, creation_parameter=True, optional=True),
         Field('entity_type', type=str, cached=True, creation_parameter=True, default='VOLUME', is_filterable=True),
-        Field('remote_pool_id', type=int, creation_parameter=True, optional=True, is_filterable=True),
-        Field('remote_replica_id', type=int, is_filterable=True, cached=True),
+        Field('remote_pool_id', type=int, creation_parameter=True, optional=True, is_filterable=True, is_sortable=True),
+        Field('remote_replica_id', type=int, is_filterable=True, is_sortable=True, cached=True),
         Field('role', type=str, cached=False, is_filterable=True),
         Field('progress', type=int),
         Field('jobs', type=list, cached=False),
         Field('job_state'),
         Field('pending_job_count', type=int),
         Field('throughput', type=int),
-        Field('restore_point', type=MillisecondsDatetimeType, is_filterable=True),
+        Field('restore_point', type=MillisecondsDatetimeType, is_filterable=True, is_sortable=True),
         Field('last_synchronized', type=MillisecondsDatetimeType),
-        Field('last_replicated_guid', api_name='_consistent_guid', is_filterable=True),
-        Field('state', type=str, cached=False),
+        Field('last_replicated_guid', api_name='_consistent_guid', is_filterable=True, is_sortable=True),
+        Field('state', type=str, cached=False, is_filterable=True),
         Field('state_description'),
         Field('state_reason'),
         Field('initial', api_name='is_initial', type=bool, cached=False),
         Field('sync_interval', api_name='sync_interval', type=MillisecondsDeltaType,
-              mutable=True, creation_parameter=True, is_filterable=True, optional=True),
-        Field('rpo', api_name='rpo_value', type=MillisecondsDeltaType, mutable=True, is_filterable=True),
+              mutable=True, creation_parameter=True, is_filterable=True, is_sortable=True, optional=True),
+        Field('rpo', api_name='rpo_value', type=MillisecondsDeltaType, mutable=True,
+              creation_parameter=True, is_filterable=True, is_sortable=True, optional=True),
         Field('rpo_state'),
-        Field('rpo_type'),
-        Field('remote_cg_id', type=int, is_filterable=True, cached=True),
-        Field('remote_cg_name', is_filterable=True),
+        Field('rpo_type', is_filterable=True, is_sortable=True),
+        Field('remote_cg_id', type=int, is_filterable=True, is_sortable=True, cached=True),
+        Field('remote_cg_name', is_filterable=True, is_sortable=True),
         Field('local_cg_id', type=int, is_filterable=True, cached=True),
-        Field('local_cg_name', is_filterable=True),
-        Field('local_pool_id', type=int, is_filterable=True),
-        Field('local_pool_name', is_filterable=True),
-        Field('remote_pool_name', is_filterable=True),
+        Field('local_cg_name', is_filterable=True, is_sortable=True),
+        Field('local_pool_id', type=int, is_filterable=True, is_sortable=True),
+        Field('local_pool_name', is_filterable=True, is_sortable=True),
+        Field('remote_pool_name', is_filterable=True, is_sortable=True),
         Field('staging_area_allocated_size', type=CapacityType),
-        Field('replication_type', type=str, creation_parameter=True, optional=True, is_filterable=True,
-              feature_name="sync_replication"),
-        Field('sync_state', type=str, feature_name="sync_replication", cached=False),
+        Field('replication_type', type=str, creation_parameter=True, optional=True,
+              is_filterable=True, is_sortable=True, feature_name="sync_replication"),
+        Field('sync_state', type=str, feature_name="sync_replication", cached=False, is_filterable=True),
         Field('sync_duration', type=int),
         Field('async_mode', type=bool, feature_name="sync_replication", cached=False),
         Field('latency', type=int, feature_name="sync_replication"),
-        Field('domino', type=bool, is_filterable=True, feature_name="sync_replication"),
+        Field('domino', type=bool, is_filterable=True, is_sortable=True, feature_name="sync_replication"),
         Field('assigned_sync_remote_ips', type=list, api_name="_assigned_sync_remote_ips",
               feature_name="sync_replication"),
         Field('next_job_start_time', type=MillisecondsDatetimeType),
         Field('next_restore_point', type=MillisecondsDatetimeType),
-        Field('permanent_failure_wait_interval', type=MillisecondsDeltaType, mutable=True),
-        Field('temporary_failure_retry_interval', type=MillisecondsDeltaType, mutable=True),
-        Field('temporary_failure_retry_count', type=int, mutable=True),
+        Field('permanent_failure_wait_interval', type=MillisecondsDeltaType, mutable=True,
+              is_filterable=True, is_sortable=True),
+        Field('temporary_failure_retry_interval', type=MillisecondsDeltaType, mutable=True,
+              is_filterable=True, is_sortable=True),
+        Field('temporary_failure_retry_count', type=int, mutable=True, is_filterable=True, is_sortable=True),
         Field('started_at', type=MillisecondsDatetimeType),
+        Field('preferred', api_name='is_preferred', type=bool, optional=True, is_filterable=True, is_sortable=True,
+              creation_parameter=True, mutable=True, feature_name="active_active_preferred_on_replica"),
+        Field('local_entity', api_name='local_entity_id', binding=ReplicaEntityBinding(), creation_parameter=True,
+              optional=True, is_filterable=True, is_sortable=True, feature_name="replica_auto_create",
+              add_getter=False),
+        Field('remote_entity', api_name='remote_entity_id', binding=ReplicaEntityBinding(), creation_parameter=True,
+              optional=True, is_filterable=True, is_sortable=True, feature_name="replica_auto_create",
+              add_getter=False),
+        Field('local_entity_name', type=str, creation_parameter=True, optional=True, is_filterable=True,
+              is_sortable=True, feature_name="replica_auto_create"),
+        Field('remote_entity_name', type=str, creation_parameter=True, optional=True, is_filterable=True,
+              is_sortable=True, feature_name="replica_auto_create"),
+        Field('base_action', type=str, creation_parameter=True, optional=True,
+              feature_name="replica_auto_create"),
+        Field('concurrent_replica', type=bool, is_filterable=True, is_sortable=True,
+              mutable=False, feature_name="concurrent_replication"),
+        Field("suspended_from_local", type=bool, is_filterable=True, is_sortable=True,
+              mutable=False, feature_name="active_active_suspend"),
     ]
 
     @classmethod
     def is_supported(cls, system):
         return system.compat.has_replication()
 
-    @classmethod
-    def _create(cls, system, url, data, tags=None):
-        if not system.compat.has_sync_replication() and data.get('replication_type', 'ASYNC').lower() == 'async':
-            # workaround to preserve older behavior for async creation
-            data.setdefault('sync_interval', 4000)
-        return super(Replica, cls)._create(system, url, data, tags)
-
     def _get_entity_collection(self):
         if self.is_filesystem():
             return self.system.filesystems
@@ -256,20 +281,28 @@ class Replica(SystemObject):
             return self.system.volumes
         return self.system.cons_groups
 
-    def get_local_entity(self):
-        """Returns the local entity used for replication, be it a volume, filesystem or a consistency group
-        """
+    def _get_entity(self, is_local, from_cache=False, safe=False):
+        system = self.system if is_local else self.get_remote_system(from_cache=from_cache, safe=safe)
+        if system is None:
+            return None
+        side = 'local' if is_local else 'remote'
+
         if self.is_consistency_group():
-            return self.system.cons_groups.get_by_id_lazy(self.get_field('local_cg_id'))
+            return system.cons_groups.get_by_id_lazy(self.get_field('{}_cg_id'.format(side)))
 
         pairs = self.get_entity_pairs(from_cache=True)
         if len(pairs) > 1:
             raise TooManyObjectsFound('Entity pairs for {}'.format(self))
         [pair] = pairs
         if self.is_filesystem():
-            return self.system.filesystems.get_by_id_lazy(pair['local_entity_id'])
+            return system.filesystems.get_by_id_lazy(pair['{}_entity_id'.format(side)])
         else:
-            return self.system.volumes.get_by_id_lazy(pair['local_entity_id'])
+            return system.volumes.get_by_id_lazy(pair['{}_entity_id'.format(side)])
+
+    def get_local_entity(self):
+        """Returns the local entity used for replication, be it a volume, filesystem or a consistency group
+        """
+        return self._get_entity(is_local=True)
 
     def _get_entity_tags(self):
         if self.is_consistency_group():
@@ -325,9 +358,10 @@ class Replica(SystemObject):
         if snapshot_id is None:
             return None
         returned = self._get_entity_collection().get_by_id_lazy(snapshot_id)
-        gossip.trigger_with_tags('infinidat.sdk.replica_snapshot_created', {'snapshot': returned}, tags=['infinibox'])
+        gossip.trigger_with_tags('infinidat.sdk.replica_snapshot_created',
+                                 {'snapshot': returned, 'replica_deleted': False},
+                                 tags=['infinibox'])
         self._notify_post_exposure(self, returned)
-        gadget.log_operation(self, "expose last consistent snapshot")
         return returned
 
 
@@ -362,12 +396,21 @@ class Replica(SystemObject):
             return self.get_local_entity().get_members().to_list()
         return [self.get_local_entity()]
 
-    def get_remote_data_entities(self):
+    def get_remote_data_entities(self, from_cache=False):
         """Returns all local volumes, whether as part of a consistency group, filesystem or a single volume
         """
         if self.is_consistency_group():
-            return self.get_remote_entity().get_members().to_list()
-        return [self.get_remote_entity()]
+            return self.get_remote_entity(from_cache=from_cache).get_members().to_list()
+        return [self.get_remote_entity(from_cache=from_cache)]
+
+    def get_remote_data_entity_from_local(self, dataset):
+        remote_system = self.get_remote_system(from_cache=True)
+        if remote_system is None:
+            return None
+        collection = remote_system.objects.get_binder_by_type_name(dataset.get_type_name())
+        for pair in self.get_entity_pairs(from_cache=True):
+            if pair['local_entity_id'] == dataset.id:
+                return collection.get_by_id_lazy(pair['remote_entity_id'])
 
     def is_consistency_group(self):
         """Returns whether this replica is configured with a consistency group as a local entity
@@ -399,7 +442,6 @@ class Replica(SystemObject):
                                          {'replica': self, 'exception': e}, tags=['infinibox'])
         gossip.trigger_with_tags('infinidat.sdk.post_replica_suspend', {'replica': self}, tags=['infinibox'])
         self.invalidate_cache('state')
-        gadget.log_operation(self, "suspend")
 
     def sync(self):
         """Starts a sync job
@@ -407,7 +449,6 @@ class Replica(SystemObject):
         returned = self.system.api.post(self.get_this_url_path().add_path('sync'),
                                         headers={'X-INFINIDAT-RAW-RESPONSE': 'true'})
         result = returned.get_result()
-        gadget.log_operation(self, "sync", params=result)
         return result
 
     def resume(self):
@@ -422,7 +463,6 @@ class Replica(SystemObject):
                                          {'replica': self, 'exception': e}, tags=['infinibox'])
         gossip.trigger_with_tags('infinidat.sdk.post_replica_resume', {'replica': self}, tags=['infinibox'])
         self.invalidate_cache('state')
-        gadget.log_operation(self, "resume")
 
     @require_sync_replication
     def switch_role(self):
@@ -437,21 +477,32 @@ class Replica(SystemObject):
                                          {'replica': self, 'exception': e}, tags=['infinibox'])
         gossip.trigger_with_tags('infinidat.sdk.post_replica_switch_role', {'replica': self}, tags=['infinibox'])
         self.invalidate_cache()
-        gadget.log_operation(self, "switch role")
 
     def is_type_sync(self):
         if self.system.compat.has_sync_replication():
             return self.get_replication_type().lower() == 'sync'
         return False
 
+    def is_type_active_active(self):
+        if self.system.compat.has_sync_replication():
+            return self.get_replication_type(from_cache=True).lower() == 'active_active'
+        return False
+
     def is_type_async(self):
         if self.system.compat.has_sync_replication():
             return self.get_replication_type().lower() == 'async'
         return True
 
-    def _is_in_sync_state(self, sync_state):
+    def _is_in_sync_state(self, *sync_states):
         current_sync_state = self.get_sync_state()
-        return current_sync_state and current_sync_state.lower() == sync_state.lower()
+        return current_sync_state and current_sync_state.lower() in sync_states
+
+    def _is_in_state(self, *states, **kwargs):
+        current_state = self.get_state(**kwargs)
+        return current_state and current_state.lower() in states
+
+    def _get_state_lower(self, **kwargs):
+        return str(self.get_state(**kwargs)).lower()
 
     def is_synchronized(self):
         """Returns True if this replica sync state is 'SYNCHRONIZED'
@@ -485,7 +536,6 @@ class Replica(SystemObject):
         :param params: Optional dictionary containing additional parameters for the type change
          """
         self._change_type(new_type='async', params=params)
-        gadget.log_operation(self, "change type to async")
 
     @require_sync_replication
     def change_type_to_sync(self, params=None):
@@ -494,7 +544,6 @@ class Replica(SystemObject):
         :param params: Optional dictionary containing additional parameters for the type change
          """
         self._change_type(new_type='sync', params=params)
-        gadget.log_operation(self, "change type to sync")
 
     def _change_type(self, new_type, params=None):
         if params is None:
@@ -522,48 +571,56 @@ class Replica(SystemObject):
         if self.is_target():
             raise CannotGetReplicaState('Replica state cannot be checked on target replica')
 
-    def is_suspended(self, *args, **kwargs):
+    def _is_in_suspended_state(self, *states, **kwargs):
+        self._validate_can_check_state()
+        return self._is_in_state(*states, **kwargs)
+
+    def is_suspended(self, **kwargs):
         """Returns whether or not this replica is currently suspended
         """
-        self._validate_can_check_state()
-        return self.get_state(*args, **kwargs).lower() in ['suspended', 'auto_suspended']
+        return self._is_in_suspended_state('suspended', 'auto_suspended', **kwargs)
 
-    def is_user_suspended(self, *args, **kwargs):
+    def is_user_suspended(self, **kwargs):
         """Returns whether or not this replica is currently suspended due to a user request
         """
-        self._validate_can_check_state()
-        return self.get_state(*args, **kwargs).lower() == 'suspended'
+        return self._is_in_suspended_state('suspended', **kwargs)
 
-    def is_idle(self, *args, **kwargs):
+    def is_auto_suspended(self, **kwargs):
+        """Returns whether or not this replica is in auto_suspended state
+        """
+        return self._is_in_suspended_state('auto_suspended', **kwargs)
+
+    def is_idle(self, **kwargs):
         """Returns whether or not this replica is in idle state
         """
         self._validate_can_check_state()
         if self.system.compat.has_sync_job_states():
             return not self.is_replicating() and not self.is_initial_replication()\
                 and not self.is_stalled() and self.is_active()
-        return self.get_state(*args, **kwargs).lower() == 'idle'
+        return self._is_in_state('idle', **kwargs)
 
-    def is_auto_suspended(self, *args, **kwargs):
-        """Returns whether or not this replica is in auto_suspended state
-        """
-        self._validate_can_check_state()
-        return self.get_state(*args, **kwargs).lower() == 'auto_suspended'
+    def is_lagging(self, **kwargs):
+        if self.is_type_active_active():
+            return self._is_in_sync_state("lagging", **kwargs)
+        return False
 
-    def is_initial_replication(self, *args, **kwargs):
+    def is_initial_replication(self, **kwargs):
         """Returns whether or not this replica is in initiating state
         """
+        if self.is_type_active_active():
+            return self._is_in_sync_state("initializing", "initializing_pending", **kwargs)
         self._validate_can_check_state()
         if self.system.compat.has_sync_job_states():
-            return self.is_initial() and self._any_sync_job_state_contains(['initializing', 'stalled'])
-        return 'initial' in self.get_state(*args, **kwargs).lower()
+            return self.is_initial(**kwargs) and self._any_sync_job_state_contains(['initializing', 'stalled'])
+        return 'initial' in self._get_state_lower(**kwargs)
 
-    def is_pending(self):
+    def is_pending(self, **kwargs):
         """Returns whether or not this replication is waiting to start initializing
         """
         self._validate_can_check_state()
         if not self.system.compat.has_sync_job_states():
             raise NotImplementedError("This system ({}) doesn't support replica \"pending\" state".format(self.system))
-        return self.is_active() and self._any_sync_job_state_contains('pending')
+        return self.is_active(**kwargs) and self._any_sync_job_state_contains('pending')
 
     def _get_jobs(self):
         returned = self.get_field('jobs')
@@ -578,27 +635,34 @@ class Replica(SystemObject):
                 j['state'] = global_job_state
         return returned
 
-    def is_replicating(self, *args, **kwargs):
+    def is_replicating(self, **kwargs):
         """Returns whether or not this replica is in replicating state
         """
         self._validate_can_check_state()
         if self.system.compat.has_sync_job_states():
             return self.is_active() and self._any_sync_job_state_contains('replicating')
-        return self.get_state(*args, **kwargs).lower() == 'replicating'
+        return self._is_in_state('replicating', **kwargs)
 
-    def is_stalled(self):
+    def is_stalled(self, **kwargs):
+        if self.is_type_active_active():
+            return self._is_in_sync_state("sync_stalled", **kwargs)
         self._validate_can_check_state()
         if not self.system.compat.has_sync_job_states():
             raise NotImplementedError("Checking for stalled is not supported on systems without sync job states")
         return self._any_sync_job_state_contains('stalled')
 
-    def is_active(self, *args, **kwargs):
+    def is_active(self, **kwargs):
         """Returns whether or not the replica is currently active
         """
+        if self.is_type_active_active():
+            return self._is_in_sync_state('synchronized', 'initializing',
+                                          'initializing_pending', 'sync_in_progress',
+                                          **kwargs)
         self._validate_can_check_state()
+        state = self._get_state_lower(**kwargs)
         if self.system.compat.has_sync_job_states():
-            return self.get_state().lower() == 'active'
-        return self.get_state(*args, **kwargs).lower() in ['idle', 'initiating', 'initial_replication', 'replicating']
+            return state == 'active'
+        return state in ['idle', 'initiating', 'initial_replication', 'replicating']
 
     def change_role(self, entity_pairs=OMIT):
         """Changes the role of this replica from source to target or vice-versa
@@ -617,11 +681,15 @@ class Replica(SystemObject):
     def is_source(self, *args, **kwargs):
         """A predicate returning whether or not the replica is currently in the "source" role
         """
+        if self.is_type_active_active():
+            return False
         return self.get_role(*args, **kwargs).lower() == 'source'
 
     def is_target(self, *args, **kwargs):
         """A predicate returning whether or not the replica is currently in the "target" role
         """
+        if self.is_type_active_active():
+            return False
         return not self.is_source(*args, **kwargs)
 
     def has_local_entity(self, entity):
@@ -631,58 +699,56 @@ class Replica(SystemObject):
                 return True
         return False
 
+    def _should_retain_staging_area(self, retain_value=OMIT):
+        if retain_value is not OMIT:
+            return retain_value
+        return self.system.compat.has_replica_auto_create()
+
     # pylint: disable=arguments-differ
-    def delete(self, retain_staging_area=False, force_if_remote_error=False, force_on_target=False,
-               force_if_no_remote_credentials=False):
+    def delete(self, retain_staging_area=OMIT, force_if_remote_error=OMIT, force_on_target=OMIT,
+               force_if_no_remote_credentials=OMIT, force_on_local=OMIT, keep_serial_on_local=OMIT):
         """Deletes this replica
         """
         path = self.get_this_url_path()
-        if retain_staging_area:
-            path = path.add_query_param('retain_staging_area', 'true')
-        if force_if_remote_error:
-            path = path.add_query_param('force_if_remote_error', 'true')
-        if force_on_target:
-            path = path.add_query_param('force_on_target', 'true')
-        if force_if_no_remote_credentials:
-            path = path.add_query_param('force_if_no_remote_credentials', 'true')
-
-        try:
-            remote_replica = self.get_remote_replica(safe=True)
-            if remote_replica is None:
-                _logger.debug('Failed to get remote replica during delete operation')
-        except UnknownSystem:
-            remote_replica = None
-
+        requested_retain_staging_area = retain_staging_area
+        remote_replica = self.get_remote_replica(safe=True)
+        if remote_replica is None:
+            _logger.debug('Failed to get remote replica during delete operation')
+        retain_staging_area = self._should_retain_staging_area(retain_value=retain_staging_area)
         if retain_staging_area:
             self._notify_pre_exposure(self)
             self._notify_pre_exposure(remote_replica)
 
-        gadget.log_entity_deletion(self)
-        with self._get_delete_context():
-            try:
-                resp = self.system.api.delete(path)
-                entity_pairs = None
-                result = resp.get_result()
-                if result:
-                    entity_pairs = result.get('entity_pairs')
-                gossip.trigger_with_tags(
-                    'infinidat.sdk.replica_deleted',
-                    {'replica': self, 'entity_pairs': entity_pairs},
-                    tags=['infinibox'])
-            except Exception as e:  # pylint: disable=broad-except
-                with end_reraise_context():
-                    if retain_staging_area:
-                        self._notify_exposure_failure(self, e)
-                        self._notify_exposure_failure(remote_replica, e)
+        try:
+            resp = self._send_delete_with_hooks_triggering(
+                path,
+                force_if_remote_error=force_if_remote_error,
+                force_on_target=force_on_target,
+                force_if_no_remote_credentials=force_if_no_remote_credentials,
+                force_on_local=force_on_local,
+                keep_serial_on_local=keep_serial_on_local,
+                retain_staging_area=requested_retain_staging_area)
+        except Exception as e:  # pylint: disable=broad-except
+            with end_reraise_context():
+                if retain_staging_area:
+                    self._notify_exposure_failure(self, e)
+                    self._notify_exposure_failure(remote_replica, e)
+
+        result = resp.get_result()
+        entity_pairs = result.get('entity_pairs') if result else None
+        gossip.trigger_with_tags('infinidat.sdk.replica_deleted',
+                                 {'replica': self, 'entity_pairs': entity_pairs, 'deletion_params': path.query_dict},
+                                 tags=['infinibox'])
 
         if retain_staging_area:
             local, remote = self._get_deletion_result(resp.get_result(), remote_replica)
         else:
             local = remote = None
         for replica, snap in (self, local), (remote_replica, remote):
-            if snap is not None:
+            if snap is not None and snap.id != 0:
                 gossip.trigger_with_tags(
-                    'infinidat.sdk.replica_snapshot_created', {'snapshot': snap}, tags=['infinibox'])
+                    'infinidat.sdk.replica_snapshot_created', {'snapshot': snap, 'replica_deleted': True},
+                    tags=['infinibox'])
 
                 self._notify_post_exposure(replica, snap)
         return local, remote
@@ -731,24 +797,21 @@ class Replica(SystemObject):
         return local, remote
 
     def get_remote_system(self, from_cache=True, safe=False):
-        return self.get_link(from_cache=from_cache).get_linked_system(safe=safe)
+        return self.get_link(from_cache=from_cache).get_linked_system(from_cache=from_cache, safe=safe)
 
     def get_remote_replica(self, from_cache=False, safe=False):
-        """Get the corresponsing replica object in the remote machine. For this to work, the SDK user should
-        call the register_related_system method of the Infinibox object when a link to a remote system is consructed
+        """Get the corresponding replica object in the remote machine. For this to work, the SDK user should
+        call the register_related_system method of the Infinibox object when a link to a remote system is constructed
         for the first time"""
-        linked_system = self.get_link(from_cache=from_cache).get_linked_system(safe=safe)
+        linked_system = self.get_remote_system(from_cache=from_cache, safe=safe)
         if linked_system is None:
             return None
         return linked_system.replicas.get_by_id_lazy(self.get_remote_replica_id(from_cache=from_cache))
 
-    def get_remote_entity(self):
+    def get_remote_entity(self, from_cache=False, safe=False):
         """Fetches the remote replicated entity if available
         """
-        peer = self.get_remote_replica()
-        if peer is None:
-            return None
-        return peer.get_local_entity()
+        return self._get_entity(is_local=False, from_cache=from_cache, safe=safe)
 
     def get_remote_entity_pairs(self):
         """Returns the entity_pairs configuration as held by the remote replica
